import { PublicKey } from "@solana/web3.js";
import * as anchor from "@project-serum/anchor";
import BN from "bn.js";
export declare type CollectionBoardingInfo = {
    name: string;
    collection_update_authority: PublicKey;
    enabled: boolean;
    verified_collection_address?: PublicKey;
    governance?: PublicKey;
    auction_house: PublicKey;
    meta_data_url: string;
    vanity_url: string;
    token_type: string;
    listing_requester: PublicKey;
    request_type: number;
};
export declare const useManageAdmin: (provider: anchor.AnchorProvider) => {
    createConfig: null;
    updateAdmin: null;
    updateFee: null;
} | {
    createConfig: (fee: BN) => Promise<[string, string]>;
    updateAdmin: (newAdminKey: PublicKey, adminConfig: PublicKey) => Promise<string>;
    updateFee: (fee: BN, adminConfig: PublicKey) => Promise<string>;
};
export declare const useAdmin: (provider: anchor.AnchorProvider, configurationKey?: PublicKey) => {
    approveListing: null;
    denyListing: null;
    setEnableListing: null;
    isAdmin: null;
} | {
    approveListing: (seed: PublicKey) => Promise<string>;
    denyListing: (seed: PublicKey) => Promise<string>;
    setEnableListing: (seed: PublicKey, isEnabled: boolean) => Promise<string>;
    isAdmin: (admin: PublicKey) => Promise<boolean>;
};
export declare const useListingRequest: (provider: anchor.AnchorProvider | null, configurationKey?: PublicKey) => {
    requestListing: null;
    requestListingRefund: null;
    updateMetadataUrl: null;
} | {
    requestListing: (collectionBoardingInfo: CollectionBoardingInfo) => Promise<[string, string]>;
    requestListingRefund: (seed: PublicKey) => Promise<string>;
    updateMetadataUrl: (seed: PublicKey, newUrl: string) => Promise<string>;
};
export declare const useListingQuery: (provider: anchor.AnchorProvider | null, configurationKey?: PublicKey) => {
    getListingRequest: null;
    getAllApprovedListings: null;
    getAllPendingListings: null;
    getApprovedListings: null;
    getPendingListings: null;
    isApproved: null;
    hasToken: null;
    getAllApprovedListingsByRequestor?: undefined;
    getAllPendingListingsByRequestor?: undefined;
} | {
    getAllApprovedListings: () => Promise<CollectionBoardingInfo[]>;
    getAllPendingListings: () => Promise<CollectionBoardingInfo[]>;
    getAllApprovedListingsByRequestor: () => Promise<CollectionBoardingInfo[]>;
    getAllPendingListingsByRequestor: () => Promise<CollectionBoardingInfo[]>;
    isApproved: (verifiedCollectionAddress: PublicKey) => Promise<boolean>;
    hasToken: (seed: PublicKey) => Promise<boolean>;
    getListingRequest?: undefined;
    getApprovedListings?: undefined;
    getPendingListings?: undefined;
};
