"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useListingQuery = exports.useListingRequest = exports.useAdmin = exports.useManageAdmin = void 0;
const web3_js_1 = require("@solana/web3.js");
const grape_collections_1 = require("../../target/types/grape_collections");
const anchor = __importStar(require("@project-serum/anchor"));
const anchor_1 = require("@project-serum/anchor");
const tokenAccountUtils_1 = require("./tokenAccountUtils");
const spl_token_1 = require("@solana/spl-token");
const bn_js_1 = __importDefault(require("bn.js"));
const utils_1 = require("./utils");
const CONFIGURATION_KEY = "Eb3pRdyww7P8ZxELwm9iTrkyQxqvk4v6aAKKKyLgWomh";
const GRAPE_MARKETPLACE_TOKEN = "72sYUZCgcWZJR1UnNgwxCJdTRAARiEm9nFm5TFB5j2Nx";
const useManageAdmin = (provider) => {
    if (!provider) {
        return {
            createConfig: null,
            updateAdmin: null,
            updateFee: null,
        };
    }
    const program = new anchor_1.Program(grape_collections_1.IDL, new web3_js_1.PublicKey(utils_1.PROGRAM_ID), provider);
    return {
        createConfig: (fee) => __awaiter(void 0, void 0, void 0, function* () {
            const configKey = anchor.web3.Keypair.generate();
            return Promise.all([
                yield program.methods
                    .initializeConfig(provider.wallet.publicKey, fee)
                    .accounts({
                    adminConfig: configKey.publicKey,
                    funder: provider.wallet.publicKey,
                    systemProgram: web3_js_1.SystemProgram.programId,
                })
                    .signers([configKey])
                    .rpc(),
                configKey.publicKey.toBase58(),
            ]);
        }),
        updateAdmin: (newAdminKey, adminConfig) => __awaiter(void 0, void 0, void 0, function* () {
            return program.methods
                .updateConfig(newAdminKey)
                .accounts({
                admin: provider.wallet.publicKey,
                adminConfig: adminConfig,
            })
                .rpc();
        }),
        updateFee: (fee, adminConfig) => __awaiter(void 0, void 0, void 0, function* () {
            return program.methods
                .setFee(fee)
                .accounts({
                admin: provider.wallet.publicKey,
                adminConfig: adminConfig,
            })
                .rpc();
        }),
    };
};
exports.useManageAdmin = useManageAdmin;
const useAdmin = (provider, configurationKey = new web3_js_1.PublicKey(CONFIGURATION_KEY)) => {
    if (!provider) {
        return {
            approveListing: null,
            denyListing: null,
            setEnableListing: null,
            isAdmin: null,
        };
    }
    const program = new anchor_1.Program(grape_collections_1.IDL, new web3_js_1.PublicKey(utils_1.PROGRAM_ID), provider);
    return {
        approveListing: (seed) => __awaiter(void 0, void 0, void 0, function* () {
            return yield (0, utils_1.approveOrDeny)(provider, program, seed, configurationKey, true);
        }),
        denyListing: (seed) => __awaiter(void 0, void 0, void 0, function* () {
            const [listingRequest, _bump] = yield web3_js_1.PublicKey.findProgramAddress([configurationKey.toBuffer(), seed.toBuffer()], new web3_js_1.PublicKey(utils_1.PROGRAM_ID));
            // Check if listingRequest account has enough SOL to provide refund
            const listingRequestAccountInfo = yield provider.connection.getAccountInfo(listingRequest);
            if (listingRequestAccountInfo == null) {
                throw Error(`Invalid listing request PDA ${listingRequest.toBase58()}`);
            }
            const listingRequestAccount = yield program.account.collectionListingRequest.fetch(listingRequest);
            const rentExemption = yield provider.connection.getMinimumBalanceForRentExemption(utils_1.COLLECTION_BOARDING_INFO_SIZE);
            if (listingRequestAccountInfo.lamports <
                listingRequestAccount.fee + rentExemption) {
                //Fund account to provide refund
                let payFeeTx = new anchor.web3.Transaction().add(web3_js_1.SystemProgram.transfer({
                    fromPubkey: provider.wallet.publicKey,
                    toPubkey: listingRequest,
                    lamports: listingRequestAccount.fee.toNumber() +
                        rentExemption -
                        listingRequestAccountInfo.lamports,
                }));
                return yield (0, utils_1.approveOrDeny)(provider, program, seed, configurationKey, false, payFeeTx);
            }
            return yield (0, utils_1.approveOrDeny)(provider, program, seed, configurationKey, false);
        }),
        setEnableListing: (seed, isEnabled) => __awaiter(void 0, void 0, void 0, function* () {
            const listingRequest = yield (0, utils_1.getListingRequestFromCollectionAddress)(seed, configurationKey);
            const listingRequestAccount = yield program.account.collectionListingRequest.fetch(listingRequest);
            let enableOrDisabletx = yield program.methods
                .enable(isEnabled)
                .accounts({
                collectionBoardingInfo: listingRequest,
                admin: provider.wallet.publicKey,
                adminConfig: configurationKey,
                listingRequestor: listingRequestAccount.listingRequestor,
            })
                .transaction();
            return provider.sendAndConfirm(enableOrDisabletx);
        }),
        isAdmin: (admin) => __awaiter(void 0, void 0, void 0, function* () {
            const config = yield program.account.config.fetch(configurationKey);
            return config.admin.equals(admin);
        }),
    };
};
exports.useAdmin = useAdmin;
const useListingRequest = (provider, configurationKey = new web3_js_1.PublicKey(CONFIGURATION_KEY)) => {
    if (!provider) {
        return {
            requestListing: null,
            requestListingRefund: null,
            updateMetadataUrl: null,
        };
    }
    const program = new anchor_1.Program(grape_collections_1.IDL, new web3_js_1.PublicKey(utils_1.PROGRAM_ID), provider);
    return {
        requestListing: (collectionBoardingInfo) => __awaiter(void 0, void 0, void 0, function* () {
            const listingRequest = yield (0, utils_1.getListingRequestFromCollectionAddress)(collectionBoardingInfo.verified_collection_address ||
                collectionBoardingInfo.collection_update_authority, configurationKey);
            // Get rent exemption listing requestor needs to pay
            const rentExemption = yield provider.connection.getMinimumBalanceForRentExemption(utils_1.COLLECTION_BOARDING_INFO_SIZE);
            // Get fee listing requestor needs to pay
            const adm = yield program.account.config.fetch(configurationKey);
            let payFeeTx = new anchor.web3.Transaction().add(web3_js_1.SystemProgram.transfer({
                fromPubkey: provider.wallet.publicKey,
                toPubkey: listingRequest,
                lamports: adm.fee.toNumber() + rentExemption,
            }));
            const initTx = yield program.methods
                .initializeListingRequest(collectionBoardingInfo.name, collectionBoardingInfo.auction_house, collectionBoardingInfo.governance || web3_js_1.PublicKey.default, collectionBoardingInfo.meta_data_url, collectionBoardingInfo.vanity_url, collectionBoardingInfo.token_type, collectionBoardingInfo.request_type)
                .accounts({
                collectionBoardingInfo: listingRequest || web3_js_1.PublicKey.default,
                listingRequestor: provider.wallet.publicKey,
                verifiedCollectionAddress: collectionBoardingInfo.verified_collection_address ||
                    web3_js_1.PublicKey.default,
                updateAuthority: collectionBoardingInfo.collection_update_authority,
                seed: collectionBoardingInfo.verified_collection_address ||
                    collectionBoardingInfo.collection_update_authority,
                adminConfig: configurationKey,
                systemProgram: web3_js_1.SystemProgram.programId,
            })
                .transaction();
            const bothTx = payFeeTx.add(initTx);
            return Promise.all([
                provider.sendAndConfirm(bothTx),
                listingRequest.toBase58(),
            ]);
        }),
        requestListingRefund: (seed) => __awaiter(void 0, void 0, void 0, function* () {
            const listingRequest = yield (0, utils_1.getListingRequestFromCollectionAddress)(seed, configurationKey);
            return yield program.methods
                .requestRefund()
                .accounts({
                listingRequestor: provider.wallet.publicKey,
                collectionBoardingInfo: listingRequest,
            })
                .rpc();
        }),
        updateMetadataUrl: (seed, newUrl) => __awaiter(void 0, void 0, void 0, function* () {
            const listingRequest = yield (0, utils_1.getListingRequestFromCollectionAddress)(seed, configurationKey);
            return yield program.methods
                .updateMetadata(newUrl)
                .accounts({
                modifier: provider.wallet.publicKey,
                collectionBoardingInfo: listingRequest,
            })
                .rpc();
        }),
    };
};
exports.useListingRequest = useListingRequest;
const useListingQuery = (provider, configurationKey = new web3_js_1.PublicKey(CONFIGURATION_KEY)) => {
    if (!provider) {
        return {
            getListingRequest: null,
            getAllApprovedListings: null,
            getAllPendingListings: null,
            getApprovedListings: null,
            getPendingListings: null,
            isApproved: null,
            hasToken: null,
        };
    }
    const program = new anchor_1.Program(grape_collections_1.IDL, new web3_js_1.PublicKey(utils_1.PROGRAM_ID), provider);
    const getLisingRequest = (seed) => __awaiter(void 0, void 0, void 0, function* () {
        const listingRequest = yield (0, utils_1.getListingRequestFromCollectionAddress)(seed, configurationKey);
        return yield program.account.collectionListingRequest.fetch(listingRequest);
    });
    return {
        //Must be approved by admin and token in listing requestor's wallet
        getAllApprovedListings: () => __awaiter(void 0, void 0, void 0, function* () {
            return (0, utils_1.accountFilter)(true, provider, program, configurationKey);
        }),
        getAllPendingListings: () => __awaiter(void 0, void 0, void 0, function* () {
            return (0, utils_1.accountFilter)(false, provider, program, configurationKey);
        }),
        getAllApprovedListingsByRequestor: () => __awaiter(void 0, void 0, void 0, function* () {
            return (0, utils_1.accountFilterByRequestor)(true, provider, program, configurationKey);
        }),
        getAllPendingListingsByRequestor: () => __awaiter(void 0, void 0, void 0, function* () {
            return (0, utils_1.accountFilterByRequestor)(false, provider, program, configurationKey);
        }),
        isApproved: (verifiedCollectionAddress) => __awaiter(void 0, void 0, void 0, function* () {
            const clr = yield getLisingRequest(verifiedCollectionAddress);
            return clr.isDaoApproved;
        }),
        hasToken: (seed) => __awaiter(void 0, void 0, void 0, function* () {
            const clr = yield getLisingRequest(seed);
            const [associatedTokenAccount, bump] = yield web3_js_1.PublicKey.findProgramAddress([
                clr.listingRequestor.toBuffer(),
                spl_token_1.TOKEN_PROGRAM_ID.toBuffer(),
                new web3_js_1.PublicKey(GRAPE_MARKETPLACE_TOKEN).toBuffer(),
            ], new web3_js_1.PublicKey(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID));
            try {
                const acct = yield provider.connection.getAccountInfo(associatedTokenAccount);
                const res = yield (0, tokenAccountUtils_1.deserializeTokenAccount)(acct.data);
                return res.amount.gte(new bn_js_1.default(1));
            }
            catch (e) {
                return false;
            }
        }),
    };
};
exports.useListingQuery = useListingQuery;
