"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.accountFilter = exports.accountFilterByRequestor = exports.approveOrDeny = exports.COLLECTION_BOARDING_INFO_SIZE = exports.getListingRequestFromCollectionAddress = exports.PROGRAM_ID = void 0;
const web3_js_1 = require("@solana/web3.js");
const bs58_1 = __importDefault(require("bs58"));
exports.PROGRAM_ID = "GArtNGsWiFYcbByxkS7MFm84Vfptt84XcweqRkcKpEEm";
const getListingRequestFromCollectionAddress = (seed, configurationKey) => __awaiter(void 0, void 0, void 0, function* () {
    const [listingRequest, _] = yield web3_js_1.PublicKey.findProgramAddress([configurationKey.toBuffer(), seed.toBuffer()], new web3_js_1.PublicKey(exports.PROGRAM_ID));
    return listingRequest;
});
exports.getListingRequestFromCollectionAddress = getListingRequestFromCollectionAddress;
exports.COLLECTION_BOARDING_INFO_SIZE = 8 +
    32 +
    32 +
    1 +
    1 +
    1 +
    32 +
    32 +
    32 +
    32 +
    8 +
    4 +
    100 +
    4 +
    200 +
    4 +
    40 +
    4 +
    200 +
    1;
const approveOrDeny = (provider, program, seed, configurationKey, approve, topOff = null) => __awaiter(void 0, void 0, void 0, function* () {
    const listingRequest = yield (0, exports.getListingRequestFromCollectionAddress)(seed, configurationKey);
    const listingRequestAccount = yield program.account.collectionListingRequest.fetch(listingRequest);
    let approveOrDenytx = yield program.methods
        .approve(approve)
        .accounts({
        collectionBoardingInfo: listingRequest,
        admin: provider.wallet.publicKey,
        adminConfig: configurationKey,
        listingRequestor: listingRequestAccount.listingRequestor,
    })
        .transaction();
    if (topOff != null) {
        return provider.sendAndConfirm(topOff.add(approveOrDenytx));
    }
    return provider.sendAndConfirm(approveOrDenytx);
});
exports.approveOrDeny = approveOrDeny;
const accountFilterByRequestor = (approved, provider, program, configurationKey) => __awaiter(void 0, void 0, void 0, function* () {
    const accounts = yield provider.connection.getParsedProgramAccounts(new web3_js_1.PublicKey(exports.PROGRAM_ID), {
        filters: [
            { dataSize: exports.COLLECTION_BOARDING_INFO_SIZE },
            {
                memcmp: {
                    offset: 8 + 32 + 32 + 1 + 1 + 1 + 32 + 32,
                    bytes: configurationKey.toBase58(),
                },
            },
            {
                memcmp: {
                    offset: 8 + 32 + 32,
                    bytes: bs58_1.default.encode([approved ? 1 : 0]),
                },
            },
            {
                memcmp: {
                    offset: 8 + 32 + 32 + 1 + 1 + 1 + 32 + 32 + 32,
                    bytes: provider.wallet.publicKey.toBase58(),
                },
            },
        ],
    });
    return Promise.all(accounts.map((acct) => __awaiter(void 0, void 0, void 0, function* () {
        const clr = yield program.account.collectionListingRequest.fetch(acct.pubkey);
        return {
            name: clr.name,
            enabled: clr.enable,
            verified_collection_address: clr.verifiedCollectionAddress,
            collection_update_authority: clr.collectionUpdateAuthority,
            auction_house: clr.auctionHouse,
            meta_data_url: clr.metaDataUrl,
            vanity_url: clr.vanityUrl,
            token_type: clr.tokenType,
            listing_requester: clr.listingRequestor,
            request_type: clr.requestType,
        };
    })));
});
exports.accountFilterByRequestor = accountFilterByRequestor;
const accountFilter = (approved, provider, program, configurationKey) => __awaiter(void 0, void 0, void 0, function* () {
    const accounts = yield provider.connection.getParsedProgramAccounts(new web3_js_1.PublicKey(exports.PROGRAM_ID), {
        filters: [
            { dataSize: exports.COLLECTION_BOARDING_INFO_SIZE },
            {
                memcmp: {
                    offset: 8 + 32 + 32 + 1 + 1 + 1 + 32 + 32,
                    bytes: configurationKey.toBase58(),
                },
            },
            {
                memcmp: {
                    offset: 8 + 32 + 32,
                    bytes: bs58_1.default.encode([approved ? 1 : 0]),
                },
            },
        ],
    });
    return Promise.all(accounts.map((acct) => __awaiter(void 0, void 0, void 0, function* () {
        const clr = yield program.account.collectionListingRequest.fetch(acct.pubkey);
        return {
            name: clr.name,
            enabled: clr.enable,
            verified_collection_address: clr.verifiedCollectionAddress,
            collection_update_authority: clr.collectionUpdateAuthority,
            auction_house: clr.auctionHouse,
            meta_data_url: clr.metaDataUrl,
            vanity_url: clr.vanityUrl,
            token_type: clr.tokenType,
            listing_requester: clr.listingRequestor,
            request_type: clr.requestType,
        };
    })));
});
exports.accountFilter = accountFilter;
